@@ -44,7 +44,7 @@ def parse_check(arg, parser):
         try:
             open(file_ + '.bai')
         except IOError as e:
-            print "couldn't find index file: ", file_ + '.bai', " check that the path is correct and that the file exists"
+            print("couldn't find index file: ", file_ + '.bai', " check that the path is correct and that the file exists")
             sys.exit(0)
 
     try:
@@ -55,19 +55,16 @@ def parse_check(arg, parser):
 
     ##
     # Error handling when parsing arguments
-    if arg.orientation == None:
-        sys.stdout.write('Warning: Orientations of library mappings not specified. Treating all libraries as fr mapped.\
-         If this is not the case, please provide orientations with the --orientations parameter.\n')
-        sys.stderr.write('Warning: Orientations of library mappings not specified. Treating all libraries as fr mapped.\
-         If this is not the case, please provide orientations with the --orientation parameter.\n')
-    elif arg.orientation != None and not len(arg.orientation) == len(arg.bamfiles):
-        sys.exit("Number of arguments doesn't match between -f and --orientation. Give one orientation for each library.\n")
+
+
+    if arg.orientation != None and not len(arg.orientation) == len(arg.bamfiles):
+        sys.exit("Number of arguments doesn't match between -f and -orientation. Give one orientation for each library.\n")
 
     elif not all([x == 'fr' or x == 'rf' for x in arg.orientation]):
-        sys.exit("'rf' and 'fr' are the only valid strings to --orinentation.\n")
+        sys.exit("'rf' and 'fr' are the only valid strings to -orinentation.\n")
 
 
-    if not all([x == None or len(x) == len(arg.bamfiles) for x in [arg.stddev , arg.mean , arg.readlen, arg.edgesupport, arg.covcutoff, arg.threshold, arg.minsize]]):
+    if not all([x == None or len(x) == len(arg.bamfiles) for x in [arg.stddev , arg.mean , arg.readlen, arg.edgesupport, arg.threshold, arg.minsize]]):
         parser.error("If any of the below following options are specified, they should have the same number of arguments as the number of BAM files.\n {0}, {1}, {2}, {3}, {4}, {5}, {6}, {7} ".format("bamfiles", "mean", "stddev", "readlen", "edgesupport",
                           "covcutoff", "threshold", "minsize"))
 
@@ -80,6 +77,20 @@ def parse_check(arg, parser):
     if not arg.bamfiles:
         parser.error("parameter -f (BAM files) is required")
 
+    if  arg.multiprocess:
+        sys.stdout.write('Warning: -q specified but will not be activated. The multiprocess option was found to contain a bug\
+         and will therefore not be activated.\n')
+        arg.multiprocess = False
+
+    if  arg.haplotype:
+        sys.stdout.write('Warning: -g (haplotype aware scaffolding) specified but this option is under construction and not stable. \
+         This option will therefore be inactivated. \n')
+        arg.haplotype = False
+
+    if (arg.bfs_traversal == True and arg.dfs_traversal == True):
+        print("Do not specify both --dfs_traversal and --bfs_traversal")
+        sys.exit(0)
+
     return()
 
 ##
