@@ -10,7 +10,7 @@ try:
     import matplotlib
     matplotlib.use('Agg')
     import matplotlib.pyplot as plt
-except ImportError, RuntimeError:
+except ImportError as RuntimeError:
     pass
 
 
@@ -26,11 +26,11 @@ def histogram(x_, param, bins=20, x_label='x', y_label='y', title='Histogram',nr
     plt.clf()
     return()
 
-def dot_plot(x_, y_, param, x_label='x', y_label='y', title='Dotplot'):
+def dot_plot(x_, y_, param, x_label='x', y_label='y', title='Dotplot', set_marker='o'):
     x = x_[:10000] # If many contigs/edges we only plot 10 000 for time and visuability purposes
     y = y_[:10000] # If many contigs/edges we only plot 10 000 for time and visuability purposes
     dest = os.path.join(param.output_directory, 'plots', title + '.png')
-    plt.scatter(x, y, marker='o')
+    plt.scatter(x, y, marker=set_marker)
     plt.xlabel(x_label)
     plt.ylabel(y_label)
     plt.title(title)
@@ -45,7 +45,7 @@ def multiple_histogram(list_of_datasets_, param, x_label='x', y_label='y', title
     list_of_datasets = [list[:10000] for list in list_of_datasets_]
     dest = os.path.join(param.output_directory, 'plots', title + '.png')
     # filter out if any of the lists contains 0 elemnets
-    list_of_datasets = filter(lambda x: len(x) > 0, list_of_datasets)
+    list_of_datasets = [x for x in list_of_datasets if len(x) > 0]
     for dataset in list_of_datasets:
         plt.hist(dataset, alpha=0.5)
     plt.xlabel(x_label)
